@@ -21,6 +21,7 @@ import shutil
 import sys
 
 from tools import building
+from tools import config
 from tools import diagnostics
 from tools import js_manipulation
 from tools import shared
@@ -328,7 +329,27 @@ def emscript(in_wasm, out_wasm, outfile_js, memfile, js_syms):
         if len(signature.params) != len(c_sig):
           diagnostics.warning('em-js-i64', 'using 64-bit arguments in EM_JS function without WASM_BIGINT is not yet fully supported: `%s` (%s, %s)', em_js_func, c_sig, signature.params)
 
+  asm_consts = create_asm_consts(metadata)
+  em_js_funcs = create_em_js(metadata)
+
   if settings.SIDE_MODULE:
+    # When building side modules, valid the EM_ASM and EM_JS string by running
+    # them through node.  Without this step, syntax errors are not surfaced
+    # until runtime.
+    # We use subprocess directly here rather than shared.check_call since
+    # check_call doesn't support the `intput` argument.
+    if asm_consts:
+      validate = '\n'.join([f'var tmp = {f};' for _, f in asm_consts])
+      proc = subprocess.run(config.NODE_JS + ['--check', '-'], input=validate.encode('utf-8'))
+      if proc.returncode:
+        exit_with_error(f'EM_ASM function validation failed (node returned {proc.returncode})')
+
+    if em_js_funcs:
+      validate = '\n'.join(em_js_funcs)
+      proc = subprocess.run(config.NODE_JS + ['--check', '-'], input=validate.encode('utf-8'))
+      if proc.returncode:
+        exit_with_error(f'EM_JS function validation failed (node returned {proc.returncode})')
+
     logger.debug('emscript: skipping remaining js glue generation')
     return
 
@@ -388,8 +409,6 @@ def emscript(in_wasm, out_wasm, outfile_js, memfile, js_syms):
     # In regular runtime, atinits etc. exist in the preamble part
     pre = apply_static_code_hooks(forwarded_json, pre)
 
-  asm_consts = create_asm_consts(metadata)
-  em_js_funcs = create_em_js(metadata)
   asm_const_pairs = ['%s: %s' % (key, value) for key, value in asm_consts]
   extra_code = ''
   if asm_const_pairs or settings.MAIN_MODULE:
@@ -601,6 +620,7 @@ def type_to_sig(type):
     webassembly.Type.I64: 'j',
     webassembly.Type.F32: 'f',
     webassembly.Type.F64: 'd',
+    webassembly.Type.EXTERNREF: 'e',
     webassembly.Type.VOID: 'v'
   }[type]
 
@@ -873,6 +893,9 @@ def create_pointer_conversion_wrappers(metadata):
     'stackAlloc': 'pp',
     'emscripten_builtin_malloc': 'pp',
     'malloc': 'pp',
+    'memalign': 'ppp',
+    'memcmp': '_ppp',
+    'memcpy': 'pppp',
     '__getTypeName': 'pp',
     'setThrew': '_p',
     'free': '_p',
@@ -891,10 +914,15 @@ def create_pointer_conversion_wrappers(metadata):
     '__main_argc_argv': '__PP',
     'emscripten_stack_set_limits': '_pp',
     '__set_stack_limits': '_pp',
+    '__set_thread_state': '_p___',
     '__cxa_can_catch': '_ppp',
     '__cxa_increment_exception_refcount': '_p',
     '__cxa_decrement_exception_refcount': '_p',
     '_wasmfs_write_file': '_ppp',
+    '_wasmfs_mknod': '_p__',
+    '_wasmfs_get_cwd': 'p_',
+    '_wasmfs_identify': '_p',
+    '_wasmfs_read_file': 'pp',
     '__dl_seterr': '_pp',
     '_emscripten_run_on_main_thread_js': '___p_',
     '_emscripten_proxy_execute_task_queue': '_p',
@@ -911,6 +939,12 @@ def create_pointer_conversion_wrappers(metadata):
     'asyncify_start_rewind': '_p',
     'asyncify_start_unwind': '_p',
     '__get_exception_message': '_ppp',
+    'stbi_image_free': 'vp',
+    'stbi_load': 'ppppp_',
+    'stbi_load_from_memory': 'pp_ppp_',
+    'emscripten_proxy_finish': '_p',
+    'emscripten_proxy_execute_queue': '_p',
+    '_emval_coro_resume': '_pp',
   }
 
   for function in settings.SIGNATURE_CONVERSIONS:
