@@ -167,18 +167,21 @@ var LibraryEmbind = {
     return errorClass;
   },
 
-
-  $createNamedFunction__deps: ['$makeLegalFunctionName'],
-  $createNamedFunction: function(name, body) {
-    name = makeLegalFunctionName(name);
-    // Use an abject with a computed property name to create a new function with
-    // a name specified at runtime, but without using `new Function` or `eval`.
-    return {
-      [name]: function() {
-        return body.apply(this, arguments);
-      }
-    }[name];
-  },
+  $createNamedFunction: (name, body) => Object.defineProperty(body, 'name', {
+    value: name
+  }),
+  // All browsers that support WebAssembly also support configurable function name,
+  // but we might be building for very old browsers via WASM2JS.
+#if MIN_CHROME_VERSION < 43 || MIN_EDGE_VERSION < 14 || MIN_SAFARI_VERSION < 100101 || MIN_FIREFOX_VERSION < 38
+  // In that case, check if configurable function name is supported at init time
+  // and, if not, replace with a fallback that returns function as-is as those browsers
+  // don't support other methods either.
+  $createNamedFunction__postset: `
+    if (!Object.getOwnPropertyDescriptor(Function.prototype, 'name').configurable) {
+      createNamedFunction = (name, body) => body;
+    }
+  `,
+#endif
 
   $embindRepr: (v) => {
     if (v === null) {
@@ -235,9 +238,7 @@ var LibraryEmbind = {
   },
 
   $getInheritedInstanceCount__deps: ['$registeredInstances'],
-  $getInheritedInstanceCount: () => {
-    return Object.keys(registeredInstances).length;
-  },
+  $getInheritedInstanceCount: () => Object.keys(registeredInstances).length,
 
   $getLiveInheritedInstances__deps: ['$registeredInstances'],
   $getLiveInheritedInstances: () => {
@@ -680,6 +681,11 @@ var LibraryEmbind = {
     });
   },
 
+  _embind_register_user_type__deps: ['_embind_register_emval'],
+  _embind_register_user_type: (rawType, name) => {
+    __embind_register_emval(rawType, name);
+  },
+
   _embind_register_memory_view__deps: ['$readLatin1String', '$registerType'],
   _embind_register_memory_view: (rawType, dataTypeIndex, name) => {
     var typeMapping = [
@@ -753,7 +759,7 @@ var LibraryEmbind = {
   // (hand-written JS code) -> (autogenerated JS invoker) -> (template-generated C++ invoker) -> (target C++ function)
   // craftInvokerFunction generates the JS invoker function for each function exposed to JS through embind.
   $craftInvokerFunction__deps: [
-    '$makeLegalFunctionName', '$runDestructors', '$throwBindingError',
+    '$createNamedFunction', '$runDestructors', '$throwBindingError',
 #if DYNAMIC_EXECUTION
     '$newFunc',
 #endif
@@ -808,7 +814,7 @@ var LibraryEmbind = {
     var argsWired = new Array(expectedArgCount);
     var invokerFuncArgs = [];
     var destructors = [];
-    return function() {
+    var invokerFn = function() {
       if (arguments.length !== expectedArgCount) {
         throwBindingError(`function ${humanName} called with ${arguments.length} arguments, expected ${expectedArgCount}`);
       }
@@ -872,7 +878,7 @@ var LibraryEmbind = {
     }
 
     var invokerFnBody = `
-      return function ${makeLegalFunctionName(humanName)}(${argsList}) {
+      return function (${argsList}) {
       if (arguments.length !== ${argCount - 2}) {
         throwBindingError('function ${humanName} called with ' + arguments.length + ' arguments, expected ${argCount - 2}');
       }`;
@@ -956,8 +962,9 @@ var LibraryEmbind = {
 
     args1.push(invokerFnBody);
 
-    return newFunc(Function, args1).apply(null, args2);
+    var invokerFn = newFunc(Function, args1).apply(null, args2);
 #endif
+    return createNamedFunction(humanName, invokerFn);
   },
 
   $embind__requireFunction__deps: ['$readLatin1String', '$throwBindingError'
@@ -995,10 +1002,11 @@ var LibraryEmbind = {
   _embind_register_function__deps: [
     '$craftInvokerFunction', '$exposePublicSymbol', '$heap32VectorToArray',
     '$readLatin1String', '$replacePublicSymbol', '$embind__requireFunction',
-    '$throwUnboundTypeError', '$whenDependentTypesAreResolved'],
+    '$throwUnboundTypeError', '$whenDependentTypesAreResolved', '$getFunctionName'],
   _embind_register_function: (name, argCount, rawArgTypesAddr, signature, rawInvoker, fn, isAsync) => {
     var argTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     name = readLatin1String(name);
+    name = getFunctionName(name);
 
     rawInvoker = embind__requireFunction(signature, rawInvoker);
 
@@ -1076,9 +1084,7 @@ var LibraryEmbind = {
         var setterArgumentType = elementTypes[i + elementsLength];
         var setter = elt.setter;
         var setterContext = elt.setterContext;
-        elt.read = (ptr) => {
-          return getterReturnType['fromWireType'](getter(getterContext, ptr));
-        };
+        elt.read = (ptr) => getterReturnType['fromWireType'](getter(getterContext, ptr));
         elt.write = (ptr, o) => {
           var destructors = [];
           setter(setterContext, ptr, setterArgumentType['toWireType'](destructors, o));
@@ -1182,10 +1188,7 @@ var LibraryEmbind = {
         var setter = field.setter;
         var setterContext = field.setterContext;
         fields[fieldName] = {
-          read: (ptr) => {
-            return getterReturnType['fromWireType'](
-                getter(getterContext, ptr));
-          },
+          read: (ptr) => getterReturnType['fromWireType'](getter(getterContext, ptr)),
           write: (ptr, o) => {
             var destructors = [];
             setter(setterContext, ptr, setterArgumentType['toWireType'](destructors, o));
@@ -1833,7 +1836,7 @@ var LibraryEmbind = {
           basePrototype = ClassHandle.prototype;
         }
 
-        var constructor = createNamedFunction(legalFunctionName, function() {
+        var constructor = createNamedFunction(name, function() {
           if (Object.getPrototypeOf(this) !== instancePrototype) {
             throw new BindingError("Use 'new' to construct " + name);
           }
@@ -1993,7 +1996,7 @@ var LibraryEmbind = {
   _embind_register_class_function__deps: [
     '$craftInvokerFunction', '$heap32VectorToArray', '$readLatin1String',
     '$embind__requireFunction', '$throwUnboundTypeError',
-    '$whenDependentTypesAreResolved'],
+    '$whenDependentTypesAreResolved', '$getFunctionName'],
   _embind_register_class_function: (rawClassType,
                                     methodName,
                                     argCount,
@@ -2005,6 +2008,7 @@ var LibraryEmbind = {
                                     isAsync) => {
     var rawArgTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     methodName = readLatin1String(methodName);
+    methodName = getFunctionName(methodName);
     rawInvoker = embind__requireFunction(invokerSignature, rawInvoker);
 
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
@@ -2127,7 +2131,7 @@ var LibraryEmbind = {
   _embind_register_class_class_function__deps: [
     '$craftInvokerFunction', '$ensureOverloadTable', '$heap32VectorToArray',
     '$readLatin1String', '$embind__requireFunction', '$throwUnboundTypeError',
-    '$whenDependentTypesAreResolved'],
+    '$whenDependentTypesAreResolved', '$getFunctionName'],
   _embind_register_class_class_function: (rawClassType,
                                           methodName,
                                           argCount,
@@ -2138,6 +2142,7 @@ var LibraryEmbind = {
                                           isAsync) => {
     var rawArgTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     methodName = readLatin1String(methodName);
+    methodName = getFunctionName(methodName);
     rawInvoker = embind__requireFunction(invokerSignature, rawInvoker);
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
       classType = classType[0];
