@@ -124,7 +124,32 @@ var LibraryEmbindShared = {
     _free(ptr);
     return rv;
   },
-
+  $getFunctionName__deps: [],
+  $getFunctionName: (signature) => {
+    signature = signature.trim();
+    const argsIndex = signature.indexOf("(");
+    if (argsIndex !== -1) {
+#if ASSERTIONS
+      assert(signature[signature.length - 1] == ")", "Parentheses for argument names should match.");
+#endif
+      return signature.substr(0, argsIndex);
+    } else {
+      return signature;
+    }
+  },
+  $getFunctionArgsName__deps: [],
+  $getFunctionArgsName: (signature) => {
+    signature = signature.trim();
+    const argsIndex = signature.indexOf("(") + 1;
+    if (argsIndex !== 0) {
+#if ASSERTIONS
+      assert(signature[signature.length - 1] == ")", "Parentheses for argument names should match.");
+#endif
+      return signature.substr(argsIndex, signature.length - argsIndex - 1).replaceAll(" ", "").split(",").filter(n => n.length);
+    } else {
+      return [];
+    }
+  },
   $heap32VectorToArray: (count, firstElement) => {
     var array = [];
     for (var i = 0; i < count; i++) {
