@@ -25,6 +25,12 @@ var LibraryEmbind = {
       this.type = type;
     }
   },
+  $UserType: class UserType {
+    constructor(typeId, name) {
+      this.typeId = typeId;
+      this.name = name;
+    }
+  },
   $FunctionDefinition: class FunctionDefinition {
     constructor(name, returnType, argumentTypes, thisType = null) {
       this.name = name;
@@ -137,11 +143,15 @@ var LibraryEmbind = {
       out.push(`export interface ${this.name}Value<T extends number> {\n`);
       out.push('  value: T;\n}\n');
       out.push(`export type ${this.name} = `);
-      const outItems = [];
-      for (const [name, value] of this.items) {
-        outItems.push(`${this.name}Value<${value}>`);
+      if (this.items.length === 0) {
+        out.push('never/* Empty Enumerator */');
+      } else {
+        const outItems = [];
+        for (const [name, value] of this.items) {
+          outItems.push(`${this.name}Value<${value}>`);
+        }
+        out.push(outItems.join('|'));
       }
-      out.push(outItems.join('|'));
       out.push(';\n\n');
     }
 
@@ -200,6 +210,10 @@ var LibraryEmbind = {
         ['bool', 'boolean'],
         ['float', 'number'],
         ['double', 'number'],
+#if WASM_BIGINT
+        ['int64_t', 'bigint'],
+        ['uint64_t', 'bigint'],
+#endif
         ['void', 'void'],
         ['std::string', jsString],
         ['std::basic_string<unsigned char>', jsString],
@@ -257,12 +271,14 @@ var LibraryEmbind = {
   $registerIntegerType: (id) => {
     registerType(id, new IntegerType(id));
   },
-  $createFunctionDefinition__deps: ['$FunctionDefinition', '$heap32VectorToArray', '$readLatin1String', '$Argument', '$whenDependentTypesAreResolved'],
+  $createFunctionDefinition__deps: ['$FunctionDefinition', '$heap32VectorToArray', '$readLatin1String', '$Argument', '$whenDependentTypesAreResolved', '$getFunctionName', '$getFunctionArgsName'],
   $createFunctionDefinition: (name, argCount, rawArgTypesAddr, hasThis, cb) => {
     const argTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     name = typeof name === 'string' ? name : readLatin1String(name);
 
-    whenDependentTypesAreResolved([], argTypes, function(argTypes) {
+    whenDependentTypesAreResolved([], argTypes, function (argTypes) {
+      const argsName = getFunctionArgsName(name);
+      name = getFunctionName(name);
       const returnType = argTypes[0];
       let thisType = null;
       let argStart = 1;
@@ -270,9 +286,15 @@ var LibraryEmbind = {
         thisType = argTypes[1];
         argStart = 2;
       }
+      if (argsName.length)
+        assert(argsName.length == (argTypes.length - hasThis - 1), 'Argument names should match number of parameters.');
       const args = [];
-      for (let i = argStart; i < argTypes.length; i++) {
-        args.push(new Argument(`_${i - argStart}`, argTypes[i]));
+      for (let i = argStart, x = 0; i < argTypes.length; i++) {
+        if (x < argsName.length) {
+          args.push(new Argument(argsName[x++], argTypes[i]));
+        } else {
+          args.push(new Argument(`_${i - argStart}`, argTypes[i]));
+        }
       }
       const funcDef = new FunctionDefinition(name, returnType, args, thisType);
       cb(funcDef);
@@ -308,6 +330,11 @@ var LibraryEmbind = {
   _embind_register_emval: (rawType, name) => {
     registerPrimitiveType(rawType, name);
   },
+  _embind_register_user_type__deps: ['$registerType', '$readLatin1String', '$UserType'],
+  _embind_register_user_type: (rawType, name) => {
+    name = readLatin1String(name);
+    registerType(rawType, new UserType(rawType, name));
+  },
   _embind_register_memory_view: (rawType, dataTypeIndex, name) => {
     // TODO
   },
@@ -581,8 +608,9 @@ var LibraryEmbind = {
   $makeLegalFunctionName: () => assert(false, 'stub function should not be called'),
   $newFunc: () => assert(false, 'stub function should not be called'),
   $runDestructors: () => assert(false, 'stub function should not be called'),
+  $createNamedFunction: () => assert(false, 'stub function should not be called'),
 };
 
-DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.push('$embindEmitTypes');
+extraLibraryFuncs.push('$embindEmitTypes');
 
 addToLibrary(LibraryEmbind);
