@@ -458,7 +458,8 @@ addToLibrary({
 #if ASYNCIFY_DEBUG
       dbg('asyncify: returnPromiseOnSuspend for', original);
 #endif
-      var type = WebAssembly.Function.type(original);
+      // TODO: remove `WebAssembly.Function.type` call when the new API is ready on all the testers.
+      var type = original.type ? original.type() : WebAssembly.Function.type(original);
       var parameters = type.parameters;
       var results = type.results;
 #if ASSERTIONS
@@ -522,16 +523,14 @@ addToLibrary({
   },
 
   emscripten_lazy_load_code__async: true,
-  emscripten_lazy_load_code: () => {
-    return Asyncify.handleSleep((wakeUp) => {
-      // Update the expected wasm binary file to be the lazy one.
-      wasmBinaryFile += '.lazy.wasm';
-      // Add a callback for when all run dependencies are fulfilled, which happens when async wasm loading is done.
-      dependenciesFulfilled = wakeUp;
-      // Load the new wasm.
-      createWasm();
-    });
-  },
+  emscripten_lazy_load_code: () => Asyncify.handleSleep((wakeUp) => {
+    // Update the expected wasm binary file to be the lazy one.
+    wasmBinaryFile += '.lazy.wasm';
+    // Add a callback for when all run dependencies are fulfilled, which happens when async wasm loading is done.
+    dependenciesFulfilled = wakeUp;
+    // Load the new wasm.
+    createWasm();
+  }),
 
   _load_secondary_module__sig: 'v',
   _load_secondary_module: async function() {
@@ -660,5 +659,5 @@ addToLibrary({
 });
 
 if (ASYNCIFY) {
-  DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.push('$Asyncify');
+  extraLibraryFuncs.push('$Asyncify');
 }
