@@ -1,3 +1,9 @@
+/**
+ * @license
+ * Copyright 2014 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
+
 // WebGLWorker worker code
 
 function WebGLBuffer(id) {
@@ -372,6 +378,7 @@ function WebGLWorker() {
   this.FLOAT_MAT3                     = 0x8B5B;
   this.FLOAT_MAT4                     = 0x8B5C;
   this.SAMPLER_2D                     = 0x8B5E;
+  this.SAMPLER_3D                     = 0x8B5F;
   this.SAMPLER_CUBE                   = 0x8B60;
   
   /* Vertex Arrays */
@@ -477,7 +484,7 @@ function WebGLWorker() {
     enabledState: {} // Stores whether various GL state via glEnable/glDisable/glIsEnabled/getParameter are enabled.
   };
   var stateDisabledByDefault = [this.BLEND, this.CULL_FACE, this.DEPTH_TEST, this.DITHER, this.POLYGON_OFFSET_FILL, this.SAMPLE_ALPHA_TO_COVERAGE, this.SAMPLE_COVERAGE, this.SCISSOR_TEST, this.STENCIL_TEST];
-  for(var i in stateDisabledByDefault) {
+  for (var i in stateDisabledByDefault) {
     bindings.enabledState[stateDisabledByDefault[i]] = false; // It will be important to distinguish between false and undefined (undefined meaning the state cap enum is unknown/unsupported).
   }
 
@@ -491,7 +498,7 @@ function WebGLWorker() {
 
   this.onmessage = function(msg) {
     //dump('worker GL got ' + JSON.stringify(msg) + '\n');
-    switch(msg.op) {
+    switch (msg.op) {
       case 'setPrefetched': {
         WebGLWorker.prototype.prefetchedParameters = msg.parameters;
         WebGLWorker.prototype.prefetchedExtensions = msg.extensions;
@@ -646,7 +653,7 @@ function WebGLWorker() {
   this.createShader = function(type) {
     var id = nextId++;
     commandBuffer.push(6, type, id);
-    return { id: id, what: 'shader', type: type };
+    return { id, what: 'shader', type };
   };
   this.deleteShader = function(shader) {
     if (!shader) return;
@@ -676,7 +683,7 @@ function WebGLWorker() {
     commandBuffer.push(12, program.id, shader.id);
   };
   this.bindAttribLocation = function(program, index, name) {
-    program.nextAttributes[name] = { what: 'attribute', name: name, size: -1, location: index, type: '?' }; // fill in size, type later
+    program.nextAttributes[name] = { what: 'attribute', name, size: -1, location: index, type: '?' }; // fill in size, type later
     program.nextAttributeVec[index] = name;
     commandBuffer.push(13, program.id, index, name);
   };
@@ -706,6 +713,7 @@ function WebGLWorker() {
         case 'mat3': return that.FLOAT_MAT3;
         case 'mat4': return that.FLOAT_MAT4;
         case 'sampler2D': return that.SAMPLER_2D;
+        case 'sampler3D': return that.SAMPLER_3D;
         case 'samplerCube': return that.SAMPLER_CUBE;
         default: throw 'not yet recognized type text: ' + text;
       }
@@ -729,7 +737,7 @@ function WebGLWorker() {
             fullname = name + '[0]';
           }
           if (!obj[name]) {
-            obj[name] = { what: type, name: fullname, size: size, location: -1, type: getTypeId(m[1]) };
+            obj[name] = { what: type, name: fullname, size, location: -1, type: getTypeId(m[1]) };
             if (vec) vec.push(name);
           }
         });
@@ -799,7 +807,7 @@ function WebGLWorker() {
     if (!(name in program.uniforms)) return null;
     var id = nextId++;
     commandBuffer.push(16, program.id, fullname, id);
-    return { what: 'location', uniform: program.uniforms[name], id: id, index: index };
+    return { what: 'location', uniform: program.uniforms[name], id, index };
   };
   this.getProgramInfoLog = function(shader) {
     return ''; // optimistic assumption of success; no proxying
@@ -859,7 +867,7 @@ function WebGLWorker() {
   };
   function duplicate(something) {
     // clone data properly: handles numbers, null, typed arrays, js arrays and array buffers
-    if (!something || typeof something === 'number') return something;
+    if (!something || typeof something == 'number') return something;
     if (something.slice) return something.slice(0); // ArrayBuffer or js array
     return new something.constructor(something); // typed array
   }
@@ -1108,6 +1116,14 @@ function WebGLWorker() {
   this.drawBuffersWEBGL = function(buffers) {
     commandBuffer.push(80, buffers);
   };
+  this.uniform1iv = function(location, data) {
+    if (!location) return;
+    commandBuffer.push(81, location.id, new Int32Array(data));
+  };
+  this.uniform1fv = function(location, data) {
+    if (!location) return;
+    commandBuffer.push(82, location.id, new Float32Array(data));
+  };
 
   // Setup
 
