@@ -17,6 +17,11 @@
 #include <emscripten/wire.h>
 #include <cstdint> // uintptr_t
 #include <vector>
+#include <type_traits>
+#if _LIBCPP_STD_VER >= 20
+#include <coroutine>
+#include <variant>
+#endif
 
 
 namespace emscripten {
@@ -30,6 +35,11 @@ namespace internal {
 template<typename WrapperType>
 val wrapped_extend(const std::string&, const val&);
 
+enum class EM_METHOD_CALLER_KIND {
+  FUNCTION = 0,
+  CONSTRUCTOR = 1,
+};
+
 // Implemented in JavaScript.  Don't call these directly.
 extern "C" {
 
@@ -61,12 +71,6 @@ EM_VAL _emval_new_u16string(const char16_t*);
 
 EM_VAL _emval_take_value(TYPEID type, EM_VAR_ARGS argv);
 
-EM_VAL _emval_new(
-    EM_VAL value,
-    unsigned argCount,
-    const TYPEID argTypes[],
-    EM_VAR_ARGS argv);
-
 EM_VAL _emval_get_global(const char* name);
 EM_VAL _emval_get_module_property(const char* name);
 EM_VAL _emval_get_property(EM_VAL object, EM_VAL key);
@@ -81,28 +85,24 @@ bool _emval_greater_than(EM_VAL first, EM_VAL second);
 bool _emval_less_than(EM_VAL first, EM_VAL second);
 bool _emval_not(EM_VAL object);
 
-EM_VAL _emval_call(
-    EM_VAL value,
-    unsigned argCount,
-    const TYPEID argTypes[],
+EM_GENERIC_WIRE_TYPE _emval_call(
+    EM_METHOD_CALLER caller,
+    EM_VAL func,
+    EM_DESTRUCTORS* destructors,
     EM_VAR_ARGS argv);
 
 // DO NOT call this more than once per signature. It will
 // leak generated function objects!
 EM_METHOD_CALLER _emval_get_method_caller(
     unsigned argCount, // including return value
-    const TYPEID argTypes[]);
+    const TYPEID argTypes[],
+    EM_METHOD_CALLER_KIND asCtor);
 EM_GENERIC_WIRE_TYPE _emval_call_method(
     EM_METHOD_CALLER caller,
     EM_VAL handle,
     const char* methodName,
     EM_DESTRUCTORS* destructors,
     EM_VAR_ARGS argv);
-void _emval_call_void_method(
-    EM_METHOD_CALLER caller,
-    EM_VAL handle,
-    const char* methodName,
-    EM_VAR_ARGS argv);
 EM_VAL _emval_typeof(EM_VAL value);
 bool _emval_instanceof(EM_VAL object, EM_VAL constructor);
 bool _emval_is_number(EM_VAL object);
@@ -111,6 +111,13 @@ bool _emval_in(EM_VAL item, EM_VAL object);
 bool _emval_delete(EM_VAL object, EM_VAL property);
 [[noreturn]] bool _emval_throw(EM_VAL object);
 EM_VAL _emval_await(EM_VAL promise);
+EM_VAL _emval_iter_begin(EM_VAL iterable);
+EM_VAL _emval_iter_next(EM_VAL iterator);
+
+#if _LIBCPP_STD_VER >= 20
+void _emval_coro_suspend(EM_VAL promise, void* coro_ptr);
+EM_VAL _emval_coro_make_promise(EM_VAL *resolve, EM_VAL *reject);
+#endif
 
 } // extern "C"
 
@@ -121,19 +128,18 @@ struct symbol_registrar {
   }
 };
 
-template<typename ReturnType, typename... Args>
+template<EM_METHOD_CALLER_KIND Kind, typename ReturnType, typename... Args>
 struct Signature {
   /*
   typedef typename BindingType<ReturnType>::WireType (*MethodCaller)(
-      EM_VAL value,
-      const char* methodName,
+      EM_VAL object,
+      EM_VAL method,
       EM_DESTRUCTORS* destructors,
       typename BindingType<Args>::WireType...);
   */
-
   static EM_METHOD_CALLER get_method_caller() {
-    constexpr WithPolicies<>::ArgTypeList<ReturnType, Args...> args;
-    thread_local EM_METHOD_CALLER mc = _emval_get_method_caller(args.getCount(), args.getTypes());
+    static constexpr WithPolicies<>::ArgTypeList<ReturnType, Args...> args;
+    thread_local EM_METHOD_CALLER mc = _emval_get_method_caller(args.getCount(), args.getTypes(), Kind);
     return mc;
   }
 };
@@ -144,7 +150,9 @@ public:
       : destructors(d)
   {}
   ~DestructorsRunner() {
-    _emval_run_destructors(destructors);
+    if (destructors) {
+      _emval_run_destructors(destructors);
+    }
   }
 
   DestructorsRunner(const DestructorsRunner&) = delete;
@@ -169,12 +177,17 @@ struct GenericWireTypeConverter<Pointee*> {
 };
 
 template<typename T>
-T fromGenericWireType(double g) {
+T fromGenericWireType(EM_GENERIC_WIRE_TYPE g) {
   typedef typename BindingType<T>::WireType WireType;
   WireType wt = GenericWireTypeConverter<WireType>::from(g);
   return BindingType<T>::fromWireType(wt);
 }
 
+template<>
+inline void fromGenericWireType<void>(EM_GENERIC_WIRE_TYPE g) {
+  (void)g;
+}
+
 template<typename... Args>
 struct PackSize;
 
@@ -264,38 +277,6 @@ private:
   std::array<GenericWireType, PackSize<Args...>::value> elements;
 };
 
-template<typename ReturnType, typename... Args>
-struct MethodCaller {
-  static ReturnType call(EM_VAL handle, const char* methodName, Args&&... args) {
-    auto caller = Signature<ReturnType, Args...>::get_method_caller();
-
-    WireTypePack<Args...> argv(std::forward<Args>(args)...);
-    EM_DESTRUCTORS destructors;
-    EM_GENERIC_WIRE_TYPE result = _emval_call_method(
-      caller,
-      handle,
-      methodName,
-      &destructors,
-      argv);
-    DestructorsRunner rd(destructors);
-    return fromGenericWireType<ReturnType>(result);
-  }
-};
-
-template<typename... Args>
-struct MethodCaller<void, Args...> {
-  static void call(EM_VAL handle, const char* methodName, Args&&... args) {
-    auto caller = Signature<void, Args...>::get_method_caller();
-
-    WireTypePack<Args...> argv(std::forward<Args>(args)...);
-    _emval_call_void_method(
-      caller,
-      handle,
-      methodName,
-      argv);
-  }
-};
-
 } // end namespace internal
 
 #define EMSCRIPTEN_SYMBOL(name)                                         \
@@ -385,48 +366,51 @@ public:
   explicit val(T&& value) {
     using namespace internal;
 
-    typedef internal::BindingType<T> BT;
     WireTypePack<T> argv(std::forward<T>(value));
-    handle = _emval_take_value(internal::TypeID<T>::get(), argv);
+    new (this) val(_emval_take_value(internal::TypeID<T>::get(), argv));
   }
 
-  val() : handle(EM_VAL(internal::_EMVAL_UNDEFINED)) {}
+  val() : val(EM_VAL(internal::_EMVAL_UNDEFINED)) {}
 
   explicit val(const char* v)
-      : handle(internal::_emval_new_cstring(v))
+      : val(internal::_emval_new_cstring(v))
   {}
 
-  val(val&& v) : handle(v.handle) {
+  // Note: unlike other constructors, this doesn't use as_handle() because
+  // it just moves a value and doesn't need to go via incref/decref.
+  // This means it's safe to move values across threads - an error will
+  // only arise if you access or free it from the wrong thread later.
+  val(val&& v) : handle(v.handle), thread(v.thread) {
     v.handle = 0;
   }
 
-  val(const val& v) : handle(v.handle) {
+  val(const val& v) : val(v.as_handle()) {
     internal::_emval_incref(handle);
   }
 
   ~val() {
-    internal::_emval_decref(handle);
+    if (handle) {
+      internal::_emval_decref(as_handle());
+      handle = 0;
+    }
   }
 
   EM_VAL as_handle() const {
+#ifdef _REENTRANT
+    assert(pthread_equal(thread, pthread_self()) && "val accessed from wrong thread");
+#endif
     return handle;
   }
 
   val& operator=(val&& v) & {
-    auto v_handle = v.handle;
-    v.handle = 0;
-    if (handle) {
-      internal::_emval_decref(handle);
-    }
-    handle = v_handle;
+    val tmp(std::move(v));
+    this->~val();
+    new (this) val(std::move(tmp));
     return *this;
   }
 
   val& operator=(const val& v) & {
-    internal::_emval_incref(v.handle);
-    internal::_emval_decref(handle);
-    handle = v.handle;
-    return *this;
+    return *this = val(v);
   }
 
   bool hasOwnProperty(const char* key) const {
@@ -434,27 +418,27 @@ public:
   }
 
   bool isNull() const {
-    return handle == EM_VAL(internal::_EMVAL_NULL);
+    return as_handle() == EM_VAL(internal::_EMVAL_NULL);
   }
 
   bool isUndefined() const {
-    return handle == EM_VAL(internal::_EMVAL_UNDEFINED);
+    return as_handle() == EM_VAL(internal::_EMVAL_UNDEFINED);
   }
 
   bool isTrue() const {
-    return handle == EM_VAL(internal::_EMVAL_TRUE);
+    return as_handle() == EM_VAL(internal::_EMVAL_TRUE);
   }
 
   bool isFalse() const {
-    return handle == EM_VAL(internal::_EMVAL_FALSE);
+    return as_handle() == EM_VAL(internal::_EMVAL_FALSE);
   }
 
   bool isNumber() const {
-    return internal::_emval_is_number(handle);
+    return internal::_emval_is_number(as_handle());
   }
 
   bool isString() const {
-    return internal::_emval_is_string(handle);
+    return internal::_emval_is_string(as_handle());
   }
 
   bool isArray() const {
@@ -462,23 +446,23 @@ public:
   }
 
   bool equals(const val& v) const {
-    return internal::_emval_equals(handle, v.handle);
+    return internal::_emval_equals(as_handle(), v.as_handle());
   }
 
   bool operator==(const val& v) const {
-    return internal::_emval_equals(handle, v.handle);
+    return equals(v);
   }
 
   bool operator!=(const val& v) const {
-    return !(*this == v);
+    return !equals(v);
   }
 
   bool strictlyEquals(const val& v) const {
-    return internal::_emval_strictly_equals(handle, v.handle);
+    return internal::_emval_strictly_equals(as_handle(), v.as_handle());
   }
 
   bool operator>(const val& v) const {
-    return internal::_emval_greater_than(handle, v.handle);
+    return internal::_emval_greater_than(as_handle(), v.as_handle());
   }
 
   bool operator>=(const val& v) const {
@@ -486,7 +470,7 @@ public:
   }
 
   bool operator<(const val& v) const {
-    return internal::_emval_less_than(handle, v.handle);
+    return internal::_emval_less_than(as_handle(), v.as_handle());
   }
 
   bool operator<=(const val& v) const {
@@ -494,39 +478,50 @@ public:
   }
 
   bool operator!() const {
-    return internal::_emval_not(handle);
-  }
-
-  template<typename... Args>
-  val new_(Args&&... args) const {
-    return internalCall(internal::_emval_new, std::forward<Args>(args)...);
+    return internal::_emval_not(as_handle());
   }
 
   template<typename T>
   val operator[](const T& key) const {
-    return val(internal::_emval_get_property(handle, val_ref(key).handle));
+    return val(internal::_emval_get_property(as_handle(), val_ref(key).as_handle()));
   }
 
   template<typename K, typename V>
   void set(const K& key, const V& value) {
-    internal::_emval_set_property(handle, val_ref(key).handle, val_ref(value).handle);
+    internal::_emval_set_property(as_handle(), val_ref(key).as_handle(), val_ref(value).as_handle());
   }
 
   template<typename T>
   bool delete_(const T& property) const {
-    return internal::_emval_delete(handle, val_ref(property).handle);
+    return internal::_emval_delete(as_handle(), val_ref(property).as_handle());
+  }
+
+  template<typename... Args>
+  val new_(Args&&... args) const {
+    using namespace internal;
+
+    return internalCall<EM_METHOD_CALLER_KIND::CONSTRUCTOR, val>(_emval_call, std::forward<Args>(args)...);
   }
 
   template<typename... Args>
   val operator()(Args&&... args) const {
-    return internalCall(internal::_emval_call, std::forward<Args>(args)...);
+    using namespace internal;
+
+    return internalCall<EM_METHOD_CALLER_KIND::FUNCTION, val>(_emval_call, std::forward<Args>(args)...);
   }
 
   template<typename ReturnValue, typename... Args>
   ReturnValue call(const char* name, Args&&... args) const {
     using namespace internal;
 
-    return MethodCaller<ReturnValue, Args...>::call(handle, name, std::forward<Args>(args)...);
+    return internalCall<EM_METHOD_CALLER_KIND::FUNCTION, ReturnValue>(
+      [name](EM_METHOD_CALLER caller,
+             EM_VAL handle,
+             EM_DESTRUCTORS* destructorsRef,
+             EM_VAR_ARGS argv) {
+        return _emval_call_method(caller, handle, name, destructorsRef, argv);
+      },
+      std::forward<Args>(args)...);
   }
 
   template<typename T, typename ...Policies>
@@ -536,9 +531,9 @@ public:
     typedef BindingType<T> BT;
     typename WithPolicies<Policies...>::template ArgTypeList<T> targetType;
 
-    EM_DESTRUCTORS destructors;
+    EM_DESTRUCTORS destructors = nullptr;
     EM_GENERIC_WIRE_TYPE result = _emval_as(
-        handle,
+        as_handle(),
         targetType.getTypes()[0],
         &destructors);
     DestructorsRunner dr(destructors);
@@ -552,7 +547,7 @@ public:
     typedef BindingType<int64_t> BT;
     typename WithPolicies<>::template ArgTypeList<int64_t> targetType;
 
-    return _emval_as_int64(handle, targetType.getTypes()[0]);
+    return _emval_as_int64(as_handle(), targetType.getTypes()[0]);
   }
 
   template<>
@@ -562,53 +557,70 @@ public:
     typedef BindingType<uint64_t> BT;
     typename WithPolicies<>::template ArgTypeList<uint64_t> targetType;
 
-    return  _emval_as_uint64(handle, targetType.getTypes()[0]);
+    return  _emval_as_uint64(as_handle(), targetType.getTypes()[0]);
   }
 
-// If code is not being compiled with GNU extensions enabled, typeof() is not a reserved keyword, so support that as a member function.
-#if __STRICT_ANSI__
-  val typeof() const {
-    return val(internal::_emval_typeof(handle));
-  }
-#endif
-
 // Prefer calling val::typeOf() over val::typeof(), since this form works in both C++11 and GNU++11 build modes. "typeof" is a reserved word in GNU++11 extensions.
   val typeOf() const {
-    return val(internal::_emval_typeof(handle));
+    return val(internal::_emval_typeof(as_handle()));
   }
 
+// If code is not being compiled with GNU extensions enabled, typeof() is a valid identifier, so support that as a member function.
+#if __is_identifier(typeof)
+  [[deprecated("Use typeOf() instead.")]]
+  val typeof() const {
+    return typeOf();
+  }
+#endif
+
   bool instanceof(const val& v) const {
-    return internal::_emval_instanceof(handle, v.handle);
+    return internal::_emval_instanceof(as_handle(), v.as_handle());
   }
 
   bool in(const val& v) const {
-    return internal::_emval_in(handle, v.handle);
+    return internal::_emval_in(as_handle(), v.as_handle());
   }
 
   [[noreturn]] void throw_() const {
-    internal::_emval_throw(handle);
+    internal::_emval_throw(as_handle());
   }
 
   val await() const {
-    return val(internal::_emval_await(handle));
+    return val(internal::_emval_await(as_handle()));
   }
 
+  struct iterator;
+
+  iterator begin() const;
+  // our iterators are sentinel-based range iterators; use nullptr as the end sentinel
+  constexpr nullptr_t end() const { return nullptr; }
+
+#if _LIBCPP_STD_VER >= 20
+  struct promise_type;
+#endif
+
 private:
-  // takes ownership, assumes handle already incref'd
+  // takes ownership, assumes handle already incref'd and lives on the same thread
   explicit val(EM_VAL handle)
-      : handle(handle)
+      : handle(handle), thread(pthread_self())
   {}
 
   template<typename WrapperType>
   friend val internal::wrapped_extend(const std::string& , const val& );
 
-  template<typename Implementation, typename... Args>
-  val internalCall(Implementation impl, Args&&... args) const {
+  template<internal::EM_METHOD_CALLER_KIND Kind, typename Ret, typename Implementation, typename... Args>
+  Ret internalCall(Implementation impl, Args&&... args) const {
     using namespace internal;
 
-    WithPolicies<>::ArgTypeList<Args...> argList;
     WireTypePack<Args...> argv(std::forward<Args>(args)...);
-    return val(impl(handle, argList.getCount(), argList.getTypes(), argv));
+    EM_DESTRUCTORS destructors = nullptr;
+    EM_GENERIC_WIRE_TYPE result = impl(
+      Signature<Kind, Ret, Args...>::get_method_caller(),
+      as_handle(),
+      &destructors,
+      argv);
+    DestructorsRunner rd(destructors);
+    return fromGenericWireType<Ret>(result);
   }
 
   template<typename T>
@@ -620,22 +632,152 @@ private:
     return v;
   }
 
+  pthread_t thread;
   EM_VAL handle;
 
   friend struct internal::BindingType<val>;
 };
 
+struct val::iterator {
+  iterator() = delete;
+  // Make sure iterator is only moveable, not copyable as it represents a mutable state.
+  iterator(iterator&&) = default;
+  iterator(const val& v) : iter(internal::_emval_iter_begin(v.as_handle())) {
+    this->operator++();
+  }
+  val&& operator*() { return std::move(cur_value); }
+  const val& operator*() const { return cur_value; }
+  void operator++() { cur_value = val(internal::_emval_iter_next(iter.as_handle())); }
+  bool operator!=(nullptr_t) const { return cur_value.as_handle() != nullptr; }
+
+private:
+  val iter;
+  val cur_value;
+};
+
+inline val::iterator val::begin() const {
+  return iterator(*this);
+}
+
+#if _LIBCPP_STD_VER >= 20
+namespace internal {
+// Awaiter defines a set of well-known methods that compiler uses
+// to drive the argument of the `co_await` operator (regardless
+// of the type of the parent coroutine).
+// This one is used for Promises represented by the `val` type.
+class val_awaiter {
+  // State machine holding awaiter's current state. One of:
+  //  - initially created with promise
+  //  - waiting with a given coroutine handle
+  //  - completed with a result
+  std::variant<val, std::coroutine_handle<val::promise_type>, val> state;
+
+  constexpr static std::size_t STATE_PROMISE = 0;
+  constexpr static std::size_t STATE_CORO = 1;
+  constexpr static std::size_t STATE_RESULT = 2;
+
+public:
+  val_awaiter(val&& promise)
+    : state(std::in_place_index<STATE_PROMISE>, std::move(promise)) {}
+
+  // just in case, ensure nobody moves / copies this type around
+  val_awaiter(val_awaiter&&) = delete;
+
+  // Promises don't have a synchronously accessible "ready" state.
+  bool await_ready() { return false; }
+
+  // On suspend, store the coroutine handle and invoke a helper that will do
+  // a rough equivalent of `promise.then(value => this.resume_with(value))`.
+  void await_suspend(std::coroutine_handle<val::promise_type> handle) {
+    internal::_emval_coro_suspend(std::get<STATE_PROMISE>(state).as_handle(), this);
+    state.emplace<STATE_CORO>(handle);
+  }
+
+  // When JS invokes `resume_with` with some value, store that value and resume
+  // the coroutine.
+  void resume_with(val&& result) {
+    auto coro = std::move(std::get<STATE_CORO>(state));
+    state.emplace<STATE_RESULT>(std::move(result));
+    coro.resume();
+  }
+
+  // `await_resume` finalizes the awaiter and should return the result
+  // of the `co_await ...` expression - in our case, the stored value.
+  val await_resume() { return std::move(std::get<STATE_RESULT>(state)); }
+};
+
+extern "C" {
+  // JS FFI helper for `val_awaiter::resume_with`.
+  void _emval_coro_resume(val_awaiter* awaiter, EM_VAL result) {
+    awaiter->resume_with(val::take_ownership(result));
+  }
+}
+}
+
+// `promise_type` is a well-known subtype with well-known method names
+// that compiler uses to drive the coroutine itself
+// (`T::promise_type` is used for any coroutine with declared return type `T`).
+class val::promise_type {
+  val promise, resolve, reject_with_current_exception;
+
+public:
+  // Create a `new Promise` and store it alongside the `resolve` and `reject`
+  // callbacks that can be used to fulfill it.
+  promise_type() {
+    EM_VAL resolve_handle;
+    EM_VAL reject_handle;
+    promise = val(internal::_emval_coro_make_promise(&resolve_handle, &reject_handle));
+    resolve = val(resolve_handle);
+    reject_with_current_exception = val(reject_handle);
+  }
+
+  // Return the stored promise as the actual return value of the coroutine.
+  val get_return_object() { return promise; }
+
+  // For similarity with JS async functions, our coroutines are eagerly evaluated.
+  auto initial_suspend() noexcept { return std::suspend_never{}; }
+  auto final_suspend() noexcept { return std::suspend_never{}; }
+
+  // On an unhandled exception, reject the stored promise instead of throwing
+  // it asynchronously where it can't be handled.
+  void unhandled_exception() {
+    reject_with_current_exception();
+  }
+
+  // Resolve the stored promise on `co_return value`.
+  template<typename T>
+  void return_value(T&& value) {
+    resolve(std::forward<T>(value));
+  }
+
+  // Return our awaiter on `co_await promise`.
+  internal::val_awaiter await_transform(val promise) {
+    return {std::move(promise)};
+  }
+};
+#endif
+
+// Declare a custom type that can be used in conjunction with
+// emscripten::register_type to emit custom TypeScript definitions for val
+// types.
+#define EMSCRIPTEN_DECLARE_VAL_TYPE(name)                                      \
+struct name : public ::emscripten::val {                                       \
+  explicit name(val const &other) : val(other) {}                              \
+};
+
 namespace internal {
 
-template<>
-struct BindingType<val> {
+template<typename T>
+struct BindingType<T, typename std::enable_if<std::is_base_of<val, T>::value &&
+                                              !std::is_const<T>::value>::type> {
   typedef EM_VAL WireType;
   static WireType toWireType(const val& v) {
-    _emval_incref(v.handle);
-    return v.handle;
+    EM_VAL handle = v.as_handle();
+    _emval_incref(handle);
+    return handle;
   }
-  static val fromWireType(WireType v) {
-    return val::take_ownership(v);
+  static T fromWireType(WireType v) {
+    return T(val::take_ownership(v));
   }
 };
 
@@ -643,11 +785,11 @@ struct BindingType<val> {
 
 template <typename T, typename... Policies>
 std::vector<T> vecFromJSArray(const val& v, Policies... policies) {
-  const size_t l = v["length"].as<size_t>();
+  const uint32_t l = v["length"].as<uint32_t>();
 
   std::vector<T> rv;
   rv.reserve(l);
-  for (size_t i = 0; i < l; ++i) {
+  for (uint32_t i = 0; i < l; ++i) {
     rv.push_back(v[i].as<T>(std::forward<Policies>(policies)...));
   }
 
