@@ -139,6 +139,7 @@ namespace wgpu {
     static constexpr uint32_t kLimitU32Undefined = WGPU_LIMIT_U32_UNDEFINED;
     static constexpr uint64_t kLimitU64Undefined = WGPU_LIMIT_U64_UNDEFINED;
     static constexpr uint32_t kMipLevelCountUndefined = WGPU_MIP_LEVEL_COUNT_UNDEFINED;
+    static constexpr uint32_t kQuerySetIndexUndefined = WGPU_QUERY_SET_INDEX_UNDEFINED;
     static constexpr size_t kWholeMapSize = WGPU_WHOLE_MAP_SIZE;
     static constexpr uint64_t kWholeSize = WGPU_WHOLE_SIZE;
 
@@ -241,11 +242,6 @@ namespace wgpu {
         Info = 0x00000002,
     };
 
-    enum class ComputePassTimestampLocation : uint32_t {
-        Beginning = 0x00000000,
-        End = 0x00000001,
-    };
-
     enum class CreatePipelineAsyncStatus : uint32_t {
         Success = 0x00000000,
         ValidationError = 0x00000001,
@@ -286,15 +282,14 @@ namespace wgpu {
         DepthClipControl = 0x00000001,
         Depth32FloatStencil8 = 0x00000002,
         TimestampQuery = 0x00000003,
-        PipelineStatisticsQuery = 0x00000004,
-        TextureCompressionBC = 0x00000005,
-        TextureCompressionETC2 = 0x00000006,
-        TextureCompressionASTC = 0x00000007,
-        IndirectFirstInstance = 0x00000008,
-        ShaderF16 = 0x00000009,
-        RG11B10UfloatRenderable = 0x0000000A,
-        BGRA8UnormStorage = 0x0000000B,
-        Float32Filterable = 0x0000000C,
+        TextureCompressionBC = 0x00000004,
+        TextureCompressionETC2 = 0x00000005,
+        TextureCompressionASTC = 0x00000006,
+        IndirectFirstInstance = 0x00000007,
+        ShaderF16 = 0x00000008,
+        RG11B10UfloatRenderable = 0x00000009,
+        BGRA8UnormStorage = 0x0000000A,
+        Float32Filterable = 0x0000000B,
     };
 
     enum class FilterMode : uint32_t {
@@ -324,14 +319,6 @@ namespace wgpu {
         Linear = 0x00000001,
     };
 
-    enum class PipelineStatisticName : uint32_t {
-        VertexShaderInvocations = 0x00000000,
-        ClipperInvocations = 0x00000001,
-        ClipperPrimitivesOut = 0x00000002,
-        FragmentShaderInvocations = 0x00000003,
-        ComputeShaderInvocations = 0x00000004,
-    };
-
     enum class PowerPreference : uint32_t {
         Undefined = 0x00000000,
         LowPower = 0x00000001,
@@ -354,8 +341,7 @@ namespace wgpu {
 
     enum class QueryType : uint32_t {
         Occlusion = 0x00000000,
-        PipelineStatistics = 0x00000001,
-        Timestamp = 0x00000002,
+        Timestamp = 0x00000001,
     };
 
     enum class QueueWorkDoneStatus : uint32_t {
@@ -365,11 +351,6 @@ namespace wgpu {
         DeviceLost = 0x00000003,
     };
 
-    enum class RenderPassTimestampLocation : uint32_t {
-        Beginning = 0x00000000,
-        End = 0x00000001,
-    };
-
     enum class RequestAdapterStatus : uint32_t {
         Success = 0x00000000,
         Unavailable = 0x00000001,
@@ -681,7 +662,7 @@ namespace wgpu {
     struct CommandBufferDescriptor;
     struct CommandEncoderDescriptor;
     struct CompilationMessage;
-    struct ComputePassTimestampWrite;
+    struct ComputePassTimestampWrites;
     struct ConstantEntry;
     struct Extent3D;
     struct InstanceDescriptor;
@@ -697,7 +678,7 @@ namespace wgpu {
     struct RenderBundleEncoderDescriptor;
     struct RenderPassDepthStencilAttachment;
     struct RenderPassDescriptorMaxDrawCount;
-    struct RenderPassTimestampWrite;
+    struct RenderPassTimestampWrites;
     struct RequestAdapterOptions;
     struct SamplerBindingLayout;
     struct SamplerDescriptor;
@@ -927,11 +908,9 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
-        void BeginPipelineStatisticsQuery(QuerySet const& querySet, uint32_t queryIndex) const;
         void DispatchWorkgroups(uint32_t workgroupCountX, uint32_t workgroupCountY = 1, uint32_t workgroupCountZ = 1) const;
         void DispatchWorkgroupsIndirect(Buffer const& indirectBuffer, uint64_t indirectOffset) const;
         void End() const;
-        void EndPipelineStatisticsQuery() const;
         void InsertDebugMarker(char const * markerLabel) const;
         void PopDebugGroup() const;
         void PushDebugGroup(char const * groupLabel) const;
@@ -1101,14 +1080,12 @@ namespace wgpu {
         using ObjectBase::operator=;
 
         void BeginOcclusionQuery(uint32_t queryIndex) const;
-        void BeginPipelineStatisticsQuery(QuerySet const& querySet, uint32_t queryIndex) const;
         void Draw(uint32_t vertexCount, uint32_t instanceCount = 1, uint32_t firstVertex = 0, uint32_t firstInstance = 0) const;
         void DrawIndexed(uint32_t indexCount, uint32_t instanceCount = 1, uint32_t firstIndex = 0, int32_t baseVertex = 0, uint32_t firstInstance = 0) const;
         void DrawIndexedIndirect(Buffer const& indirectBuffer, uint64_t indirectOffset) const;
         void DrawIndirect(Buffer const& indirectBuffer, uint64_t indirectOffset) const;
         void End() const;
         void EndOcclusionQuery() const;
-        void EndPipelineStatisticsQuery() const;
         void ExecuteBundles(size_t bundleCount, RenderBundle const * bundles) const;
         void InsertDebugMarker(char const * markerLabel) const;
         void PopDebugGroup() const;
@@ -1322,10 +1299,10 @@ namespace wgpu {
         uint64_t utf16Length;
     };
 
-    struct ComputePassTimestampWrite {
+    struct ComputePassTimestampWrites {
         QuerySet querySet;
-        uint32_t queryIndex;
-        ComputePassTimestampLocation location;
+        uint32_t beginningOfPassWriteIndex;
+        uint32_t endOfPassWriteIndex;
     };
 
     struct ConstantEntry {
@@ -1421,8 +1398,6 @@ namespace wgpu {
         char const * label = nullptr;
         QueryType type;
         uint32_t count;
-        PipelineStatisticName const * pipelineStatistics;
-        size_t pipelineStatisticsCount = 0;
     };
 
     struct QueueDescriptor {
@@ -1467,10 +1442,10 @@ namespace wgpu {
         alignas(kFirstMemberAlignment) uint64_t maxDrawCount = 50000000;
     };
 
-    struct RenderPassTimestampWrite {
+    struct RenderPassTimestampWrites {
         QuerySet querySet;
-        uint32_t queryIndex;
-        RenderPassTimestampLocation location;
+        uint32_t beginningOfPassWriteIndex;
+        uint32_t endOfPassWriteIndex;
     };
 
     struct RequestAdapterOptions {
@@ -1628,8 +1603,7 @@ namespace wgpu {
     struct ComputePassDescriptor {
         ChainedStruct const * nextInChain = nullptr;
         char const * label = nullptr;
-        size_t timestampWriteCount = 0;
-        ComputePassTimestampWrite const * timestampWrites;
+        ComputePassTimestampWrites const * timestampWrites = nullptr;
     };
 
     struct DepthStencilState {
@@ -1746,8 +1720,7 @@ namespace wgpu {
         RenderPassColorAttachment const * colorAttachments;
         RenderPassDepthStencilAttachment const * depthStencilAttachment = nullptr;
         QuerySet occlusionQuerySet = nullptr;
-        size_t timestampWriteCount = 0;
-        RenderPassTimestampWrite const * timestampWrites;
+        RenderPassTimestampWrites const * timestampWrites = nullptr;
     };
 
     struct VertexState {
