@@ -20,7 +20,8 @@ using namespace emscripten;
 using namespace internal;
 
 extern "C" {
-const char* EMSCRIPTEN_KEEPALIVE __getTypeName(const std::type_info* ti) {
+
+const char* __getTypeName(const std::type_info* ti) {
   if (has_unbound_type_names) {
 #ifdef USE_CXA_DEMANGLE
     int stat;
@@ -51,7 +52,7 @@ const char* EMSCRIPTEN_KEEPALIVE __getTypeName(const std::type_info* ti) {
 
 static InitFunc* init_funcs = nullptr;
 
-EMSCRIPTEN_KEEPALIVE void _embind_initialize_bindings() {
+void _embind_initialize_bindings() {
   for (auto* f = init_funcs; f; f = f->next) {
     f->init_func();
   }
@@ -119,7 +120,8 @@ EMSCRIPTEN_BINDINGS(builtin) {
 
   _embind_register_void(TypeID<void>::get(), "void");
 
-  _embind_register_bool(TypeID<bool>::get(), "bool", sizeof(bool), true, false);
+  _embind_register_bool(TypeID<bool>::get(), "bool", true, false);
+  static_assert(sizeof(bool) == 1);
 
   register_integer<char>("char");
   register_integer<signed char>("signed char");
@@ -173,10 +175,9 @@ EMSCRIPTEN_BINDINGS(builtin) {
   register_memory_view<uint16_t>("emscripten::memory_view<uint16_t>");
   register_memory_view<int32_t>("emscripten::memory_view<int32_t>");
   register_memory_view<uint32_t>("emscripten::memory_view<uint32_t>");
+  register_memory_view<int64_t>("emscripten::memory_view<int64_t>");
+  register_memory_view<uint64_t>("emscripten::memory_view<uint64_t>");
 
   register_memory_view<float>("emscripten::memory_view<float>");
   register_memory_view<double>("emscripten::memory_view<double>");
-#if __SIZEOF_LONG_DOUBLE__ == __SIZEOF_DOUBLE__
-  register_memory_view<long double>("emscripten::memory_view<long double>");
-#endif
 }
