@@ -1,76 +1,66 @@
+/*
+ * Copyright 2018 The Emscripten Authors.  All rights reserved.
+ * Emscripten is available under two separate licenses, the MIT license and the
+ * University of Illinois/NCSA Open Source License.  Both these licenses can be
+ * found in the LICENSE file.
+ */
 #include <emscripten/threading.h>
-#include <emscripten.h>
+#include <emscripten/console.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
-#include "webgl1.h"
-#include "webgl1_ext.h"
-#include "webgl2.h"
+#include <webgl/webgl1.h>
+#include <webgl/webgl1_ext.h>
+#include <webgl/webgl2.h>
+
+#include "webgl_internal.h"
 
 #if defined(__EMSCRIPTEN_PTHREADS__) && defined(__EMSCRIPTEN_OFFSCREEN_FRAMEBUFFER__)
 
-extern EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_do_create_context(const char *target, const EmscriptenWebGLContextAttributes *attributes);
-extern EMSCRIPTEN_RESULT emscripten_webgl_make_context_current_calling_thread(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context);
-extern EMSCRIPTEN_RESULT emscripten_webgl_do_commit_frame(void);
-extern EM_BOOL emscripten_supports_offscreencanvas(void);
-extern EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_do_get_current_context(void);
-extern void _emscripten_proxied_gl_context_activated_from_main_browser_thread(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context);
-
 pthread_key_t currentActiveWebGLContext;
 pthread_key_t currentThreadOwnsItsWebGLContext;
 static pthread_once_t tlsInit = PTHREAD_ONCE_INIT;
 
-static void InitWebGLTls()
-{
+static void InitWebGLTls() {
   pthread_key_create(&currentActiveWebGLContext, NULL);
   pthread_key_create(&currentThreadOwnsItsWebGLContext, NULL);
 }
 
-EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_create_context(const char *target, const EmscriptenWebGLContextAttributes *attributes)
-{
+EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_create_context(const char *target, const EmscriptenWebGLContextAttributes *attributes) {
   GL_FUNCTION_TRACE(__func__);
-  if (!attributes)
-  {
-    EM_ASM(console.error('emscripten_webgl_create_context: attributes pointer is null!'));
+  if (!attributes) {
+    emscripten_err("emscripten_webgl_create_context: attributes pointer is null!");
     return 0;
   }
   pthread_once(&tlsInit, InitWebGLTls);
 
   if (attributes->proxyContextToMainThread == EMSCRIPTEN_WEBGL_CONTEXT_PROXY_ALWAYS ||
-    (attributes->proxyContextToMainThread == EMSCRIPTEN_WEBGL_CONTEXT_PROXY_FALLBACK && !emscripten_supports_offscreencanvas()))
-  {
+    (attributes->proxyContextToMainThread == EMSCRIPTEN_WEBGL_CONTEXT_PROXY_FALLBACK && !emscripten_supports_offscreencanvas())) {
     EmscriptenWebGLContextAttributes attrs = *attributes;
     attrs.renderViaOffscreenBackBuffer = EM_TRUE;
     return (EMSCRIPTEN_WEBGL_CONTEXT_HANDLE)emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_III, &emscripten_webgl_do_create_context, target, &attrs);
-  }
-  else
-  {
+  } else {
     return emscripten_webgl_do_create_context(target, attributes);
   }
 }
 
-EMSCRIPTEN_RESULT emscripten_webgl_make_context_current(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context)
-{
+EMSCRIPTEN_RESULT emscripten_webgl_make_context_current(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context) {
   GL_FUNCTION_TRACE(__func__);
-  if (emscripten_webgl_get_current_context() == context) return;
+  if (emscripten_webgl_get_current_context() == context)
+    return EMSCRIPTEN_RESULT_SUCCESS;
 
   void *owningThread = *(void**)(context + 4);
-  if (owningThread == pthread_self())
-  {
+  if (owningThread == pthread_self()) {
     EMSCRIPTEN_RESULT r = emscripten_webgl_make_context_current_calling_thread(context);
-    if (r == EMSCRIPTEN_RESULT_SUCCESS)
-    {
+    if (r == EMSCRIPTEN_RESULT_SUCCESS) {
       pthread_setspecific(currentActiveWebGLContext, (void*)context);
       pthread_setspecific(currentThreadOwnsItsWebGLContext, (void*)1);
     }
     return r;
-  }
-  else
-  {
+  } else {
     EMSCRIPTEN_RESULT r = emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_II, &emscripten_webgl_make_context_current_calling_thread, context);
-    if (r == EMSCRIPTEN_RESULT_SUCCESS)
-    {
+    if (r == EMSCRIPTEN_RESULT_SUCCESS) {
       pthread_setspecific(currentActiveWebGLContext, (void*)context);
       pthread_setspecific(currentThreadOwnsItsWebGLContext, (void*)0);
       _emscripten_proxied_gl_context_activated_from_main_browser_thread(context);
@@ -79,13 +69,11 @@ EMSCRIPTEN_RESULT emscripten_webgl_make_context_current(EMSCRIPTEN_WEBGL_CONTEXT
   }
 }
 
-EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_get_current_context(void)
-{
+EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_get_current_context(void) {
   return (EMSCRIPTEN_WEBGL_CONTEXT_HANDLE)pthread_getspecific(currentActiveWebGLContext);
 }
 
-EMSCRIPTEN_RESULT EMSCRIPTEN_KEEPALIVE emscripten_webgl_commit_frame(void)
-{
+EMSCRIPTEN_RESULT emscripten_webgl_commit_frame(void) {
   GL_FUNCTION_TRACE(__func__);
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
     return emscripten_webgl_do_commit_frame();
@@ -93,8 +81,7 @@ EMSCRIPTEN_RESULT EMSCRIPTEN_KEEPALIVE emscripten_webgl_commit_frame(void)
     return (EMSCRIPTEN_RESULT)emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_I, &emscripten_webgl_do_commit_frame);
 }
 
-static void *memdup(const void *ptr, size_t sz)
-{
+static void *memdup(const void *ptr, size_t sz) {
   if (!ptr) return 0;
   void *dup = malloc(sz);
   if (dup) memcpy(dup, ptr, sz);
@@ -114,48 +101,42 @@ ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glBlendEquationSeparate, GLenum, GLen
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glBlendFunc, GLenum, GLenum);
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glBlendFuncSeparate, GLenum, GLenum, GLenum, GLenum);
 
-void glBufferData(GLenum target, GLsizeiptr size, const void *data, GLenum usage)
-{
+void glBufferData(GLenum target, GLsizeiptr size, const void *data, GLenum usage) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glBufferData(target, size, data, usage);
-  else
-  {
-    if (size < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(data, size);
-      if (ptr || !data) // glBufferData(data=0) can always be handled asynchronously
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferData, ptr, target, size, ptr, usage);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glBufferData, target, size, data, usage);
+    return;
   }
+
+  if (size < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(data, size);
+    if (ptr || !data) { // glBufferData(data=0) can always be handled asynchronously
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferData, ptr, target, size, ptr, usage);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glBufferData, target, size, data, usage);
 }
 
-void glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, const void *data)
-{
+void glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, const void *data) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glBufferSubData(target, offset, size, data);
-  else
-  {
-    if (size < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(data, size);
-      if (ptr || !data)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, ptr, target, offset, size, ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, target, offset, size, data);
+    return;
   }
+
+  if (size < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(data, size);
+    if (ptr || !data) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, ptr, target, offset, size, ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, target, offset, size, data);
 }
 
 RET_SYNC_GL_FUNCTION_1(EM_FUNC_SIG_II, GLenum, glCheckFramebufferStatus, GLenum);
@@ -255,11 +236,9 @@ ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glStencilMaskSeparate, GLenum, GLuint
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glStencilOp, GLenum, GLenum, GLenum);
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glStencilOpSeparate, GLenum, GLenum, GLenum, GLenum);
 
-static ssize_t ImageSize(int width, int height, GLenum format, GLenum type)
-{
+static ssize_t ImageSize(int width, int height, GLenum format, GLenum type) {
   int numChannels;
-  switch(format)
-  {
+  switch (format) {
     case GL_ALPHA: case GL_LUMINANCE: case GL_DEPTH_COMPONENT: case GL_RED: case GL_RED_INTEGER: numChannels = 1; break;
     case GL_RG: case GL_RG_INTEGER: numChannels = 2; break;
     case GL_RGB: case 0x8C40/*GL_SRGB_EXT*/: case GL_RGB_INTEGER: numChannels = 3; break;
@@ -267,8 +246,7 @@ static ssize_t ImageSize(int width, int height, GLenum format, GLenum type)
     default: return -1;
   }
   int sizePerPixel;
-  switch(type)
-  {
+  switch (type) {
     case GL_UNSIGNED_BYTE: case GL_BYTE: sizePerPixel = numChannels; break;
     case GL_UNSIGNED_SHORT: case 0x8D61/*GL_HALF_FLOAT_OES*/: case GL_HALF_FLOAT: case GL_SHORT: case GL_UNSIGNED_SHORT_5_6_5: case GL_UNSIGNED_SHORT_4_4_4_4: case GL_UNSIGNED_SHORT_5_5_5_1: sizePerPixel = numChannels*2; break;
     case GL_UNSIGNED_INT: case GL_FLOAT: case GL_INT: case GL_UNSIGNED_INT_5_9_9_9_REV: case GL_UNSIGNED_INT_2_10_10_10_REV: case GL_UNSIGNED_INT_10F_11F_11F_REV: case GL_UNSIGNED_INT_24_8: sizePerPixel = numChannels*4; break;
@@ -277,27 +255,24 @@ static ssize_t ImageSize(int width, int height, GLenum format, GLenum type)
   return width*height*sizePerPixel;
 }
 
-void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const void *pixels)
-{
+void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const void *pixels) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels);
-  else
-  {
-    ssize_t sz = ImageSize(width, height, format, type);
-    if (!pixels || (sz >= 0 && sz < 256*1024)) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(pixels, sz);
-      if (ptr || !pixels)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, ptr, target, level, internalformat, width, height, border, format, type, ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, target, level, internalformat, width, height, border, format, type, pixels);
+    return;
   }
+
+  ssize_t sz = ImageSize(width, height, format, type);
+  if (!pixels || (sz >= 0 && sz < 256*1024)) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(pixels, sz);
+    if (ptr || !pixels) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, ptr, target, level, internalformat, width, height, border, format, type, ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, target, level, internalformat, width, height, border, format, type, pixels);
 }
 
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIF, void, glTexParameterf, GLenum, GLenum, GLfloat);
@@ -305,293 +280,254 @@ VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glTexParameterfv, GLenum, GLenum
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glTexParameteri, GLenum, GLenum, GLint);
 VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glTexParameteriv, GLenum, GLenum, const GLint *);
 
-void glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const void *pixels)
-{
+void glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const void *pixels) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels);
-  else
-  {
-    ssize_t sz = ImageSize(width, height, format, type);
-    if (!pixels || (sz >= 0 && sz < 256*1024)) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(pixels, sz);
-      if (ptr || !pixels)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, ptr, target, level, xoffset, yoffset, width, height, format, type, ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, target, level, xoffset, yoffset, width, height, format, type, pixels);
+    return;
   }
+
+  ssize_t sz = ImageSize(width, height, format, type);
+  if (!pixels || (sz >= 0 && sz < 256*1024)) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(pixels, sz);
+    if (ptr || !pixels) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, ptr, target, level, xoffset, yoffset, width, height, format, type, ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, target, level, xoffset, yoffset, width, height, format, type, pixels);
 }
 
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIF, void, glUniform1f, GLint, GLfloat);
 
-void glUniform1fv(GLint location, GLsizei count, const GLfloat *value)
-{
+void glUniform1fv(GLint location, GLsizei count, const GLfloat *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform1fv(location, count, value);
-  else
-  {
-    size_t sz = sizeof(GLfloat)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, ptr, location, count, (GLfloat*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, location, count, value);
+    return;
   }
+
+  size_t sz = sizeof(GLfloat)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, ptr, location, count, (GLfloat*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, location, count, value);
 }
 
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glUniform1i, GLint, GLint);
 
-void glUniform1iv(GLint location, GLsizei count, const GLint *value)
-{
+void glUniform1iv(GLint location, GLsizei count, const GLint *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform1iv(location, count, value);
-  else
-  {
-    size_t sz = sizeof(GLint)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, ptr, location, count, (GLint*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, location, count, value);
+    return;
   }
+
+  size_t sz = sizeof(GLint)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, ptr, location, count, (GLint*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, location, count, value);
 }
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIFF, void, glUniform2f, GLint, GLfloat, GLfloat);
 
-void glUniform2fv(GLint location, GLsizei count, const GLfloat *value)
-{
+void glUniform2fv(GLint location, GLsizei count, const GLfloat *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform2fv(location, count, value);
-  else
-  {
-    size_t sz = 2*sizeof(GLfloat)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, ptr, location, count, (GLfloat*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, location, count, value);
+    return;
   }
+
+  size_t sz = 2*sizeof(GLfloat)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, ptr, location, count, (GLfloat*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, location, count, value);
 }
 
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glUniform2i, GLint, GLint, GLint);
 
-void glUniform2iv(GLint location, GLsizei count, const GLint *value)
-{
+void glUniform2iv(GLint location, GLsizei count, const GLint *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform2iv(location, count, value);
-  else
-  {
-    size_t sz = 2*sizeof(GLint)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, ptr, location, count, (GLint*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, location, count, value);
+    return;
   }
+
+  size_t sz = 2*sizeof(GLint)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, ptr, location, count, (GLint*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, location, count, value);
 }
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIFFF, void, glUniform3f, GLint, GLfloat, GLfloat, GLfloat);
 
-void glUniform3fv(GLint location, GLsizei count, const GLfloat *value)
-{
+void glUniform3fv(GLint location, GLsizei count, const GLfloat *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform3fv(location, count, value);
-  else
-  {
-    size_t sz = 3*sizeof(GLfloat)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, ptr, location, count, (GLfloat*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, location, count, value);
+    return;
   }
+
+  size_t sz = 3*sizeof(GLfloat)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, ptr, location, count, (GLfloat*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, location, count, value);
 }
 
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glUniform3i, GLint, GLint, GLint, GLint);
 
-void glUniform3iv(GLint location, GLsizei count, const GLint *value)
-{
+void glUniform3iv(GLint location, GLsizei count, const GLint *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform3iv(location, count, value);
-  else
-  {
-    size_t sz = 3*sizeof(GLint)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, ptr, location, count, (GLint*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, location, count, value);
+    return;
   }
+
+  size_t sz = 3*sizeof(GLint)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, ptr, location, count, (GLint*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, location, count, value);
 }
 ASYNC_GL_FUNCTION_5(EM_FUNC_SIG_VIFFFF, void, glUniform4f, GLint, GLfloat, GLfloat, GLfloat, GLfloat);
 
-void glUniform4fv(GLint location, GLsizei count, const GLfloat *value)
-{
+void glUniform4fv(GLint location, GLsizei count, const GLfloat *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform4fv(location, count, value);
-  else
-  {
-    size_t sz = 4*sizeof(GLfloat)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, ptr, location, count, (GLfloat*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, location, count, value);
+    return;
   }
+
+  size_t sz = 4*sizeof(GLfloat)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, ptr, location, count, (GLfloat*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, location, count, value);
 }
 
 ASYNC_GL_FUNCTION_5(EM_FUNC_SIG_VIIIII, void, glUniform4i, GLint, GLint, GLint, GLint, GLint);
 
-void glUniform4iv(GLint location, GLsizei count, const GLint *value)
-{
+void glUniform4iv(GLint location, GLsizei count, const GLint *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform4iv(location, count, value);
-  else
-  {
-    size_t sz = 4*sizeof(GLint)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, ptr, location, count, (GLint*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, location, count, value);
+    return;
   }
+
+  size_t sz = 4*sizeof(GLint)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, ptr, location, count, (GLint*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, location, count, value);
 }
 
-void glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value)
-{
+void glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniformMatrix2fv(location, count, transpose, value);
-  else
-  {
-    size_t sz = 2*2*sizeof(GLfloat)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, ptr, location, count, transpose, (GLfloat*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, location, count, transpose, value);
+    return;
   }
+  size_t sz = 2*2*sizeof(GLfloat)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, ptr, location, count, transpose, (GLfloat*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, location, count, transpose, value);
 }
 
-void glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value)
-{
+void glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniformMatrix3fv(location, count, transpose, value);
-  else
-  {
-    size_t sz = 3*3*sizeof(GLfloat)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, ptr, location, count, transpose, (GLfloat*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, location, count, transpose, value);
+    return;
   }
+  size_t sz = 3*3*sizeof(GLfloat)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, ptr, location, count, transpose, (GLfloat*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, location, count, transpose, value);
 }
 
-void glUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value)
-{
+void glUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) {
   GL_FUNCTION_TRACE(__func__);
-  if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
+  if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniformMatrix4fv(location, count, transpose, value);
-  else
-  {
-    size_t sz = 4*4*sizeof(GLfloat)*count;
-    if (sz < 256*1024) // run small buffer sizes asynchronously by copying - large buffers run synchronously
-    {
-      void *ptr = memdup(value, sz);
-      if (ptr)
-      {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, ptr, location, count, transpose, (GLfloat*)ptr);
-        return;
-      }
-      // Fall through on allocation failure and run synchronously.
-    }
-
-    emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, location, count, transpose, value);
+    return;
   }
+  size_t sz = 4*4*sizeof(GLfloat)*count;
+  if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
+    void *ptr = memdup(value, sz);
+    if (ptr) {
+      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, ptr, location, count, transpose, (GLfloat*)ptr);
+      return;
+    }
+    // Fall through on allocation failure and run synchronously.
+  }
+
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, location, count, transpose, value);
 }
 
 ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VI, void, glUseProgram, GLuint);
@@ -630,8 +566,7 @@ RET_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetQueryObjectui64vEXT, GLenum,
 // Returns a function pointer to the given WebGL 1 extension function, when queried without
 // a GL extension suffix such as "EXT", "OES", or "ANGLE". This function is used by
 // emscripten_GetProcAddress() to implement legacy GL emulation semantics for portability.
-void *_webgl1_match_ext_proc_address_without_suffix(const char *name)
-{
+void *_webgl1_match_ext_proc_address_without_suffix(const char *name) {
   RETURN_FN_WITH_SUFFIX(glGenQueries, EXT);
   RETURN_FN_WITH_SUFFIX(glDeleteQueries, EXT);
   RETURN_FN_WITH_SUFFIX(glIsQuery, EXT);
@@ -668,8 +603,7 @@ void *_webgl1_match_ext_proc_address_without_suffix(const char *name)
   return 0;
 }
 
-void *emscripten_webgl1_get_proc_address(const char *name)
-{
+void *emscripten_webgl1_get_proc_address(const char *name) {
   RETURN_FN(glActiveTexture);
   RETURN_FN(glAttachShader);
   RETURN_FN(glBindAttribLocation);
