@@ -10,7 +10,7 @@
 // Parameters:
 //    setting file.  Can specify 'settings.js' here, alternatively create a temp
 //                   file with modified settings and supply the filename here.
-//    shell file     This is the file that will be processed by the preprocessor
+//    input file     This is the file that will be processed by the preprocessor
 
 'use strict';
 
@@ -18,14 +18,14 @@ const fs = require('fs');
 const path = require('path');
 global.vm = require('vm');
 
-const arguments_ = process['argv'].slice(2);
+const arguments_ = process.argv.slice(2);
 const debug = false;
 
-global.print = function(x) {
-  process['stdout'].write(x + '\n');
+global.print = (x) => {
+  process.stdout.write(x + '\n');
 };
-global.printErr = function(x) {
-  process['stderr'].write(x + '\n');
+global.printErr = (x) => {
+  process.stderr.write(x + '\n');
 };
 
 global.assert = require('assert');
@@ -41,17 +41,17 @@ function find(filename) {
   return filename;
 }
 
-global.read = function(filename) {
+global.read = (filename) => {
   const absolute = find(filename);
   return fs.readFileSync(absolute).toString();
 };
 
-global.load = function(f) {
+global.load = (f) => {
   (0, eval)(read(f) + '//# sourceURL=' + find(f));
 };
 
 const settingsFile = arguments_[0];
-const shellFile = arguments_[1];
+const inputFile = arguments_[1];
 const expandMacros = arguments_.includes('--expandMacros');
 
 load(settingsFile);
@@ -59,6 +59,5 @@ load('utility.js');
 load('modules.js');
 load('parseTools.js');
 
-const toHTML = expandMacros ? processMacros(preprocess(shellFile)) : preprocess(shellFile);
-
-print(toHTML);
+const output = expandMacros ? processMacros(preprocess(inputFile)) : preprocess(inputFile);
+process.stdout.write(output);
